@@ -4,19 +4,19 @@ import inspect
 from tkinter import messagebox
 from typing import Any, Callable, Dict, Optional, Sequence, Tuple, Type, Union
 
+import ttkthemes
 from matplotlib.backends import _backend_tk
 from scipy import signal
-import ttkthemes
 
 from sample.widgets import logging
 from sample.widgets import responsive as tk
-from sample.widgets import sample, userfiles, utils
+from sample.widgets import userfiles, utils
 
 
 # --- Parsers ----------------------------------------------------------------
 def try_func(func: Callable,
              exc: Union[Type[Exception], Tuple[Type[Exception], ...]],
-             default: Optional = None):
+             default: Optional[Any] = None):
   """Function wrapper for returning a default value on fail
 
   Args:
@@ -73,17 +73,43 @@ def strip_time_parse(x: str) -> Optional[float]:
   return x
 
 
-def non_negative(x: str) -> float:
-  """Clip floats at zero. Invalid values are also mapped to zero
+def float_clip(left: Optional[float] = None,
+               right: Optional[float] = None,
+               default: Optional[float] = None):
+  """Clip floats between extremities.
   Args:
-    x (str): Input number as a string
+    left (float): Left extremum
+    right (float): Right extremum
+    default (float): Default value on error
+
   Returns:
-    float: :data:`x` if positive, else :data:`0`"""
-  try:
-    x = float(x)
-  except ValueError:
-    return 0
-  return max(0., x)
+    float: Clipped value"""
+  if default is None:
+    if left is None:
+      if right is None:
+        default = 0
+      else:
+        default = right
+    elif right is None:
+      default = left
+    else:
+      default = (left + right) / 2
+
+  def float_clip_(x: str,
+                  left: Optional[float] = left,
+                  right: Optional[float] = right,
+                  default: float = default):
+    try:
+      x = float(x)
+    except ValueError:
+      return default
+    if left is not None:
+      x = max(left, x)
+    if right is not None:
+      x = min(right, x)
+    return x
+
+  return float_clip_
 
 
 def custom_positive_int(x: str, dflt: Optional[int] = 1) -> int:
@@ -130,27 +156,27 @@ def postprocess_fbound(smfb_0: float,
   elif smfb_0 == smfb_1:
     smfb_0 = 20
     smfb_1 = 16000
-  in_kw = dict(
-      smfb_0=smfb_0,
-      smfb_1=smfb_1,
-  )
-  out_kw = dict(sinusoidal_model__frequency_bounds=(smfb_0, smfb_1))
+  in_kw = {
+      "smfb_0": smfb_0,
+      "smfb_1": smfb_1,
+  }
+  out_kw = {"sinusoidal__tracker__frequency_bounds": (smfb_0, smfb_1)}
   return in_kw, out_kw
 
 
-def postprocess_windows(sinusoidal_model__n: int, wsize: int,
+def postprocess_windows(sinusoidal__n: int, wsize: int,
                         wtype: str) -> Tuple[Dict[str, Any], Dict[str, Any]]:
   """Postprocess frequency bounds
 
   Args:
-    sinusoidal_model__n (int): FFT size
+    sinusoidal__n (int): FFT size
     wsize (int): FFT window size
     wtype (str): FFT window type
 
   Returns:
     dict, dict: Postprocessed settings and parameters as dictionaries"""
   w = None
-  wsize = min(wsize, sinusoidal_model__n)
+  wsize = min(wsize, sinusoidal__n)
   while True:
     try:
       w = signal.get_window(window=wtype, Nx=wsize)
@@ -162,10 +188,23 @@ def postprocess_windows(sinusoidal_model__n: int, wsize: int,
       continue
     else:
       break
-  in_kw = dict(sinusoidal_model__n=sinusoidal_model__n,
-               wsize=wsize,
-               wtype=wtype)
-  out_kw = dict(sinusoidal_model__n=sinusoidal_model__n, sinusoidal_model__w=w)
+  in_kw = {"sinusoidal__n": sinusoidal__n, "wsize": wsize, "wtype": wtype}
+  out_kw = {"sinusoidal__n": sinusoidal__n, "sinusoidal__w": w}
+  return in_kw, out_kw
+
+
+def postprocess_multiprocessing(
+    parallel: bool, n_jobs: int) -> Tuple[Dict[str, Any], Dict[str, Any]]:
+  """Postprocess number of jobs
+
+  Args:
+    parallel (bool): Toggle on/off multiprocessing
+    n_jobs (int): Number of workers
+
+  Returns:
+    dict, dict: Postprocessed settings and parameters as dictionaries"""
+  in_kw = {"parallel": parallel, "n_jobs": n_jobs}
+  out_kw = {"n_jobs": n_jobs if parallel else 0}
   return in_kw, out_kw
 
 
@@ -181,124 +220,189 @@ def postprocess_guitheme(
   if not userfiles.UserTtkTheme.is_valid(
       gui_theme, log=True, messagebox=len(gui_theme)):
     gui_theme = ""
-  return dict(gui_theme=gui_theme), {}
+  return {"gui_theme": gui_theme}, {}
 
 
 # ----------------------------------------------------------------------------
 
 _settings = (
-    ("resynth_group", dict(label="Resynthesis", is_spacer=True)),
-    ("max_n_modes",
-     dict(label="n modes",
-          get_fn=functools.partial(custom_positive_int, dflt=None),
-          init_value=None,
-          tooltip="Maximum number of modes to use for resynthesis")),
-    ("analysis_space", dict(is_spacer=True, label=" ")),
-    ("analysis_group", dict(label="Analysis", is_spacer=True)),
-    ("sinusoidal_model__max_n_sines",
-     dict(label="n sines",
-          get_fn=custom_positive_int,
-          init_value=64,
-          tooltip="Maximum number of sinusoidal tracks per frame")),
-    ("sinusoidal_model__n",
-     dict(label="fft size",
-          get_fn=next_power_of_2,
-          init_value=4096,
-          tooltip="FFT size (in bins)")),
-    ("sinusoidal_model__h",
-     dict(label="hop size",
-          get_fn=custom_positive_int,
-          init_value=1024,
-          tooltip="FTT analysis window hop size (in samples)")),
-    ("wsize",
-     dict(
-         label="window size",
-         get_fn=custom_positive_int,
-         init_value=4096,
-         tooltip="FFT analysis window size (in samples)",
-     )),
-    ("wtype",
-     dict(
-         label="window type",
-         init_value="blackman",
-         options=sorted(
-             ("boxcar", "triang", "blackman", "hamming", "hann", "bartlett",
-              "flattop", "parzen", "bohman", "blackmanharris", "nuttall",
-              "barthann", "cosine", "exponential", "tukey", "taylor")),
-         tooltip="FFT analysis window type",
-     )),
-    ("sinusoidal_model__freq_dev_offset",
-     dict(
-         label="frequency deviation offset",
-         get_fn=try_float,
-         init_value=20,
-         tooltip="Frequency deviation threshold at 0 Hz (in Hertz)",
-     )),
-    ("sinusoidal_model__freq_dev_slope",
-     dict(label="frequency deviation slope",
-          get_fn=try_float,
-          init_value=.0025,
-          tooltip="Slope of frequency deviation threshold")),
-    ("smfb_0",
-     dict(
-         label="lower frequency bound",
-         get_fn=try_float,
-         init_value=20,
-         tooltip="Minimum and accepted mean frequency (in Hertz)",
-     )),
-    ("smfb_1",
-     dict(
-         label="upper frequency bound",
-         get_fn=try_float,
-         init_value=16000,
-         tooltip="Maximum and accepted mean frequency (in Hertz)",
-     )),
-    ("sinusoidal_model__peak_threshold",
-     dict(
-         label="onset threshold",
-         get_fn=try_float,
-         init_value=-66,
-         tooltip="Minimum peak magnitude for modal tracks "
-         "(magnitude at time=0, in dB)",
-     )),
-    ("sinusoidal_model__t",
-     dict(label="peak detection threshold",
-          get_fn=try_float,
-          init_value=-90,
-          tooltip="Threshold in dB for the peak detection algorithm")),
-    ("sinusoidal_model__min_sine_dur",
-     dict(label="minimum sine duration",
-          get_fn=non_negative,
-          init_value=0.1,
-          tooltip="Minimum duration of a track (in seconds)")),
-    ("sinusoidal_model__strip_t",
-     dict(
-         label="strip time",
-         get_fn=strip_time_parse,
-         init_value=0.5,
-         tooltip="Strip time (in seconds). Tracks starting later "
-         "than this time will be omitted from the track "
-         "list. If is None, then don't strip",
-     )),
-    ("sinusoidal_model__reverse",
-     dict(
-         label="reverse",
-         init_value=True,
-         boolean=True,
-         tooltip="If True, then process audio in reverse order of time",
-     )),
-    ("gui_space", dict(is_spacer=True, label=" ")),
-    ("gui_group", dict(label="GUI", is_spacer=True)),
-    ("gui_theme",
-     dict(
-         label="gui theme",
-         init_value="",
-         options=ttkthemes.THEMES,
-         tooltip="GUI theme",
-     )),
+    ("resynth_group", {
+        "label": "Resynthesis",
+        "is_spacer": True
+    }),
+    ("max_n_modes", {
+        "label": "n modes",
+        "get_fn": functools.partial(custom_positive_int, dflt=None),
+        "init_value": None,
+        "tooltip": "Maximum number of modes to use for resynthesis"
+    }),
+    ("analysis_space", {
+        "is_spacer": True,
+        "label": " "
+    }),
+    ("analysis_group", {
+        "label": "Analysis",
+        "is_spacer": True
+    }),
+    ("sinusoidal__tracker__max_n_sines", {
+        "label": "n sines",
+        "get_fn": custom_positive_int,
+        "init_value": 64,
+        "tooltip": "Maximum number of sinusoidal tracks per frame"
+    }),
+    ("sinusoidal__n", {
+        "label": "fft size",
+        "get_fn": next_power_of_2,
+        "init_value": 4096,
+        "tooltip": "FFT size (in bins)"
+    }),
+    ("sinusoidal__tracker__h", {
+        "label": "hop size",
+        "get_fn": custom_positive_int,
+        "init_value": 1024,
+        "tooltip": "FTT analysis window hop size (in samples)"
+    }),
+    ("wsize", {
+        "label": "window size",
+        "get_fn": custom_positive_int,
+        "init_value": 4096,
+        "tooltip": "FFT analysis window size (in samples)",
+    }),
+    ("wtype", {
+        "label":
+            "window type",
+        "init_value":
+            "blackman",
+        "options":
+            sorted(
+                ("boxcar", "triang", "blackman", "hamming", "hann", "bartlett",
+                 "flattop", "parzen", "bohman", "blackmanharris", "nuttall",
+                 "barthann", "cosine", "exponential", "tukey", "taylor")),
+        "tooltip":
+            "FFT analysis window type",
+    }),
+    ("sinusoidal__tracker__freq_dev_offset", {
+        "label": "frequency deviation offset",
+        "get_fn": try_float,
+        "init_value": 20,
+        "tooltip": "Frequency deviation threshold at 0 Hz (in Hertz)",
+    }),
+    ("sinusoidal__tracker__freq_dev_slope", {
+        "label": "frequency deviation slope",
+        "get_fn": try_float,
+        "init_value": 0.0025,
+        "tooltip": "Slope of frequency deviation threshold"
+    }),
+    ("smfb_0", {
+        "label": "lower frequency bound",
+        "get_fn": try_float,
+        "init_value": 20,
+        "tooltip": "Minimum and accepted mean frequency (in Hertz)",
+    }),
+    ("smfb_1", {
+        "label": "upper frequency bound",
+        "get_fn": try_float,
+        "init_value": 16000,
+        "tooltip": "Maximum and accepted mean frequency (in Hertz)",
+    }),
+    ("sinusoidal__tracker__peak_threshold", {
+        "label": "onset threshold",
+        "get_fn": try_float,
+        "init_value": -66,
+        "tooltip": "Minimum peak magnitude for modal tracks "
+                   "(magnitude at time=0, in dB)",
+    }),
+    ("sinusoidal__t", {
+        "label": "peak detection threshold",
+        "get_fn": try_float,
+        "init_value": -90,
+        "tooltip": "Threshold in dB for the peak detection algorithm"
+    }),
+    ("sinusoidal__tracker__min_sine_dur", {
+        "label": "minimum sine duration",
+        "get_fn": float_clip(left=0, default=0.1),
+        "init_value": 0.1,
+        "tooltip": "Minimum duration of a track (in seconds)"
+    }),
+    ("sinusoidal__tracker__strip_t", {
+        "label": "strip time",
+        "get_fn": strip_time_parse,
+        "init_value": 0.5,
+        "tooltip": "Strip time (in seconds). Tracks starting later "
+                   "than this time will be omitted from the track "
+                   "list. If is None, then don't strip",
+    }),
+    ("sinusoidal__tracker__reverse", {
+        "label": "reverse",
+        "init_value": True,
+        "boolean": True,
+        "tooltip": "If True, then process audio in reverse order of time",
+    }),
+    ("sinusoidal__padded", {
+        "label": "zero-padding",
+        "init_value": False,
+        "boolean": True,
+        "tooltip": "If True, then zero-pad the audio to center the first "
+                   "window on the first sample",
+    }),
+    ("beatsdrop_space", {
+        "is_spacer": True,
+        "label": " "
+    }),
+    ("beatsdrop_group", {
+        "label": "BeatsDROP",
+        "is_spacer": True
+    }),
+    ("beatsdrop", {
+        "label": "BeatsDROP",
+        "init_value": False,
+        "boolean": True,
+        "tooltip": "If True, apply BeatsDROP to de-couple beats",
+    }),
+    ("parallel", {
+        "label": "multiprocessing",
+        "init_value": False,
+        "boolean": True,
+        "tooltip": "If True, parallelize BeatsDROP "
+                   "amongst multiple processes",
+    }),
+    ("n_jobs", {
+        "label": "n jobs",
+        "init_value": 4,
+        "get_fn": functools.partial(custom_positive_int, dflt=4),
+        "tooltip": "Number of parallel processes to use",
+    }),
+    ("beat_decisor__th", {
+        "label": "threshold",
+        "get_fn": float_clip(left=0, default=4),
+        "init_value": 4,
+        "tooltip": "Frequency difference threshold"
+    }),
+    ("beatsdrop__lpf", {
+        "label": "lpf",
+        "get_fn": float_clip(left=2, right=50, default=12),
+        "init_value": 12,
+        "tooltip": "Cutoff frequency for the low-pass filter used "
+                   "for computing the amplitude autocorrelation"
+    }),
+    ("gui_space", {
+        "is_spacer": True,
+        "label": " "
+    }),
+    ("gui_group", {
+        "label": "GUI",
+        "is_spacer": True
+    }),
+    ("gui_theme", {
+        "label": "gui theme",
+        "init_value": "",
+        "options": ttkthemes.THEMES,
+        "tooltip": "GUI theme",
+    }),
 )
 
 _postprocess = (
+    postprocess_multiprocessing,
     postprocess_windows,
     postprocess_fbound,
     postprocess_guitheme,
@@ -343,7 +447,7 @@ class SettingsTab(utils.DataOnRootMixin, tk.Frame):
         tooltip: Optional[str] = None,
         get_fn: Optional[Callable] = None,
         set_fn: Optional[Callable] = None,
-        init_value: Optional = None,
+        init_value: Optional[Any] = None,
         options: Optional[Sequence[str]] = None,
         boolean: bool = False,
     ):
@@ -435,7 +539,7 @@ class SettingsTab(utils.DataOnRootMixin, tk.Frame):
     self.button.bind("<Button-1>", self.apply_cbk)
     self.button.grid()
 
-    self.sample_object = sample.SAMPLE4GUI()
+    self.sample_object_kwargs = {}
     self.apply_cbk(from_file=True)
 
   def reset_selections(self, *args, **kwargs):  # pylint: disable=W0613
@@ -503,8 +607,7 @@ class SettingsTab(utils.DataOnRootMixin, tk.Frame):
       self.settings_file.save_json(settings, indent=2)
     for k, v in settings.items():
       self._settings[k].set(v)
-    self.sample_object.set_params(**params)
-    logging.debug("SAMPLE: %s", self.sample_object)
+    self.sample_object_kwargs = params
     if prev_theme != ttk_theme.get():
       logging.info("Reload GUI to apply changes")
       if messagebox.askyesno(
@@ -513,13 +616,12 @@ class SettingsTab(utils.DataOnRootMixin, tk.Frame):
       ):
         if self.audio_cache_file.is_valid():
           logging.info("Caching state to %s", self.audio_cache_file.path)
-          self.audio_cache_file.save_pickled(
-              dict(
-                  audio_x=self.audio_x,
-                  audio_sr=self.audio_sr,
-                  audio_trim_start=self.audio_trim_start,
-                  audio_trim_stop=self.audio_trim_stop,
-              ))
+          self.audio_cache_file.save_pickled({
+              "audio_x": self.audio_x,
+              "audio_sr": self.audio_sr,
+              "audio_trim_start": self.audio_trim_start,
+              "audio_trim_stop": self.audio_trim_stop,
+          })
         r = utils.get_root(self)
         r.master.should_reload = True
         r.quit()
